# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.pagination import AsyncPager, SyncPager
from ..core.request_options import RequestOptions
from ..types.create_driver_request_attributes import CreateDriverRequestAttributes
from ..types.driver import Driver
from ..types.driver_carrier_settings import DriverCarrierSettings
from ..types.driver_has_driving_features_hidden import DriverHasDrivingFeaturesHidden
from ..types.driver_has_vehicle_unpinning_enabled import DriverHasVehicleUnpinningEnabled
from ..types.driver_hos_setting import DriverHosSetting
from ..types.driver_profile_image_base_64 import DriverProfileImageBase64
from ..types.driver_profile_image_url import DriverProfileImageUrl
from ..types.driver_remote_signout_post_driver_remote_signout_response_body import (
    DriverRemoteSignoutPostDriverRemoteSignoutResponseBody,
)
from ..types.driver_response import DriverResponse
from ..types.list_drivers_response import ListDriversResponse
from ..types.update_driver_request_attributes import UpdateDriverRequestAttributes
from ..types.update_driver_request_hos_setting import UpdateDriverRequestHosSetting
from ..types.us_driver_ruleset_override import UsDriverRulesetOverride
from .raw_client import AsyncRawDriversClient, RawDriversClient
from .types.create_driver_request_locale import CreateDriverRequestLocale
from .types.list_drivers_request_driver_activation_status import ListDriversRequestDriverActivationStatus
from .types.update_driver_request_driver_activation_status import UpdateDriverRequestDriverActivationStatus
from .types.update_driver_request_locale import UpdateDriverRequestLocale

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class DriversClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawDriversClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawDriversClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawDriversClient
        """
        return self._raw_client

    def list(
        self,
        *,
        driver_activation_status: typing.Optional[ListDriversRequestDriverActivationStatus] = None,
        limit: typing.Optional[int] = None,
        after: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        attribute_value_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        attributes: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        updated_after_time: typing.Optional[str] = None,
        created_after_time: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[Driver, ListDriversResponse]:
        """
        Get all drivers in organization.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Drivers** under the Drivers category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        driver_activation_status : typing.Optional[ListDriversRequestDriverActivationStatus]
            If value is `deactivated`, only drivers that are deactivated will appear in the response. This parameter will default to `active` if not provided (fetching only active drivers).

        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 512 objects.

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        parent_tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        attribute_value_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of attribute value IDs. Only entities associated with ALL of the referenced values will be returned (i.e. the intersection of the sets of entities with each value). Example: `attributeValueIds=076efac2-83b5-47aa-ba36-18428436dcac,6707b3f0-23b9-4fe3-b7be-11be34aea544`

        attributes : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data to return entities having given attributes using name-value pair or range query (only for numeric attributes), separated by semicolon. Only entities associated with ALL of the referenced values will be returned (i.e. the intersection of the sets of entities with each value). Example: `attributes=ExampleAttributeName:some_value&attributes=SomeOtherAttr:123&attributes=someNumericAttribute:range(10,20)`

        updated_after_time : typing.Optional[str]
            A filter on data to have an updated at time after or equal to this specified time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        created_after_time : typing.Optional[str]
            A filter on data to have a created at time after or equal to this specified time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[Driver, ListDriversResponse]
            List of all driver objects.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        response = client.drivers.list()
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(
            driver_activation_status=driver_activation_status,
            limit=limit,
            after=after,
            parent_tag_ids=parent_tag_ids,
            tag_ids=tag_ids,
            attribute_value_ids=attribute_value_ids,
            attributes=attributes,
            updated_after_time=updated_after_time,
            created_after_time=created_after_time,
            request_options=request_options,
        )

    def create(
        self,
        *,
        name: str,
        password: str,
        username: str,
        attributes: typing.Optional[typing.Sequence[CreateDriverRequestAttributes]] = OMIT,
        carrier_settings: typing.Optional[DriverCarrierSettings] = OMIT,
        current_id_card_code: typing.Optional[str] = OMIT,
        eld_adverse_weather_exemption_enabled: typing.Optional[bool] = OMIT,
        eld_big_day_exemption_enabled: typing.Optional[bool] = OMIT,
        eld_day_start_hour: typing.Optional[int] = OMIT,
        eld_exempt: typing.Optional[bool] = OMIT,
        eld_exempt_reason: typing.Optional[str] = OMIT,
        eld_pc_enabled: typing.Optional[bool] = OMIT,
        eld_ym_enabled: typing.Optional[bool] = OMIT,
        external_ids: typing.Optional[typing.Dict[str, str]] = OMIT,
        has_driving_features_hidden: typing.Optional[DriverHasDrivingFeaturesHidden] = OMIT,
        has_vehicle_unpinning_enabled: typing.Optional[DriverHasVehicleUnpinningEnabled] = OMIT,
        hos_setting: typing.Optional[DriverHosSetting] = OMIT,
        license_number: typing.Optional[str] = OMIT,
        license_state: typing.Optional[str] = OMIT,
        locale: typing.Optional[CreateDriverRequestLocale] = OMIT,
        notes: typing.Optional[str] = OMIT,
        peer_group_tag_id: typing.Optional[str] = OMIT,
        phone: typing.Optional[str] = OMIT,
        profile_image_base_64: typing.Optional[DriverProfileImageBase64] = OMIT,
        profile_image_url: typing.Optional[DriverProfileImageUrl] = OMIT,
        static_assigned_vehicle_id: typing.Optional[str] = OMIT,
        tachograph_card_number: typing.Optional[str] = OMIT,
        tag_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        us_driver_ruleset_override: typing.Optional[UsDriverRulesetOverride] = OMIT,
        vehicle_group_tag_id: typing.Optional[str] = OMIT,
        waiting_time_duty_status_enabled: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DriverResponse:
        """
        Add a driver to the organization.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Drivers** under the Drivers category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        name : str
            Driver's name.

        password : str
            Password that the driver can use to login to the Samsara driver app.

        username : str
            Driver's login username into the driver app. The username may not contain spaces or the '@' symbol. The username must be unique.

        attributes : typing.Optional[typing.Sequence[CreateDriverRequestAttributes]]

        carrier_settings : typing.Optional[DriverCarrierSettings]

        current_id_card_code : typing.Optional[str]
            The ID Card Code on the back of the physical card assigned to the driver.  Contact Samsara if you would like to enable this feature.

        eld_adverse_weather_exemption_enabled : typing.Optional[bool]
            Flag indicating this driver may use Adverse Weather exemptions in ELD logs.

        eld_big_day_exemption_enabled : typing.Optional[bool]
            Flag indicating this driver may use Big Day exemption in ELD logs.

        eld_day_start_hour : typing.Optional[int]
            `0` indicating midnight-to-midnight ELD driving hours, `12` to indicate noon-to-noon driving hours.

        eld_exempt : typing.Optional[bool]
            Flag indicating this driver is exempt from the Electronic Logging Mandate.

        eld_exempt_reason : typing.Optional[str]
            Reason that this driver is exempt from the Electronic Logging Mandate (see eldExempt).

        eld_pc_enabled : typing.Optional[bool]
            Flag indicating this driver may select the Personal Conveyance duty status in ELD logs.

        eld_ym_enabled : typing.Optional[bool]
            Flag indicating this driver may select the Yard Move duty status in ELD logs.

        external_ids : typing.Optional[typing.Dict[str, str]]
            The [external IDs](https://developers.samsara.com/docs/external-ids) for the given object.

        has_driving_features_hidden : typing.Optional[DriverHasDrivingFeaturesHidden]

        has_vehicle_unpinning_enabled : typing.Optional[DriverHasVehicleUnpinningEnabled]

        hos_setting : typing.Optional[DriverHosSetting]

        license_number : typing.Optional[str]
            Driver's state issued license number. The combination of this number and `licenseState` must be unique.

        license_state : typing.Optional[str]
            Abbreviation of US state, Canadian province, or US territory that issued driver's license.

        locale : typing.Optional[CreateDriverRequestLocale]
            Locale override (uncommon). These are specified by ISO 3166-2 country codes for supported locales. Valid values: `us`, `at`, `be`, `ca`, `gb`, `fr`, `de`, `ie`, `it`, `lu`, `mx`, `nl`, `es`, `ch`, `pr`.

        notes : typing.Optional[str]
            Notes about the driver.

        peer_group_tag_id : typing.Optional[str]
            The peer group tag id this driver belong to, used for gamification.

        phone : typing.Optional[str]
            Phone number of the driver.

        profile_image_base_64 : typing.Optional[DriverProfileImageBase64]

        profile_image_url : typing.Optional[DriverProfileImageUrl]

        static_assigned_vehicle_id : typing.Optional[str]
            ID of vehicle that the driver is permanently assigned to. (uncommon).

        tachograph_card_number : typing.Optional[str]
            Driver's assigned tachograph card number (Europe specific)

        tag_ids : typing.Optional[typing.Sequence[str]]
            IDs of tags the driver is associated with. If your access to the API is scoped by one or more tags, this field is required to pass in.

        timezone : typing.Optional[str]
            Home terminal timezone, in order to indicate what time zone should be used to calculate the ELD logs. Driver timezones use [IANA timezone database](https://www.iana.org/time-zones) keys (e.g. `America/Los_Angeles`, `America/New_York`, `Europe/London`, etc.). You can find a mapping of common timezone formats to IANA timezone keys [here](https://unicode.org/cldr/charts/latest/supplemental/zone_tzid.html).

        us_driver_ruleset_override : typing.Optional[UsDriverRulesetOverride]

        vehicle_group_tag_id : typing.Optional[str]
            Tag ID which determines which vehicles a driver will see when selecting vehicles.

        waiting_time_duty_status_enabled : typing.Optional[bool]
            Flag indicating this driver may select waiting time duty status in ELD logs

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DriverResponse
            Newly created driver object, with Samsara-generated ID.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.drivers.create(
            name="Susan Jones",
            password="aSecurePassword1234",
            username="SusanJones",
        )
        """
        _response = self._raw_client.create(
            name=name,
            password=password,
            username=username,
            attributes=attributes,
            carrier_settings=carrier_settings,
            current_id_card_code=current_id_card_code,
            eld_adverse_weather_exemption_enabled=eld_adverse_weather_exemption_enabled,
            eld_big_day_exemption_enabled=eld_big_day_exemption_enabled,
            eld_day_start_hour=eld_day_start_hour,
            eld_exempt=eld_exempt,
            eld_exempt_reason=eld_exempt_reason,
            eld_pc_enabled=eld_pc_enabled,
            eld_ym_enabled=eld_ym_enabled,
            external_ids=external_ids,
            has_driving_features_hidden=has_driving_features_hidden,
            has_vehicle_unpinning_enabled=has_vehicle_unpinning_enabled,
            hos_setting=hos_setting,
            license_number=license_number,
            license_state=license_state,
            locale=locale,
            notes=notes,
            peer_group_tag_id=peer_group_tag_id,
            phone=phone,
            profile_image_base_64=profile_image_base_64,
            profile_image_url=profile_image_url,
            static_assigned_vehicle_id=static_assigned_vehicle_id,
            tachograph_card_number=tachograph_card_number,
            tag_ids=tag_ids,
            timezone=timezone,
            us_driver_ruleset_override=us_driver_ruleset_override,
            vehicle_group_tag_id=vehicle_group_tag_id,
            waiting_time_duty_status_enabled=waiting_time_duty_status_enabled,
            request_options=request_options,
        )
        return _response.data

    def post_driver_remote_signout(
        self, *, driver_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> DriverRemoteSignoutPostDriverRemoteSignoutResponseBody:
        """
        Sign out a driver from the Samsara Driver App

        To access this endpoint, your organization must have the Samsara Platform Premier license.

        Note: Sign out requests made while a logged-in driver does not have internet connection will not log the driver out. A success response will still be provided and the driver will be logged out once they have internet connection.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Driver Remote Signout** under the Drivers category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        driver_id : str
            ID of the driver.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DriverRemoteSignoutPostDriverRemoteSignoutResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.drivers.post_driver_remote_signout(
            driver_id="12434",
        )
        """
        _response = self._raw_client.post_driver_remote_signout(driver_id=driver_id, request_options=request_options)
        return _response.data

    def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> DriverResponse:
        """
        Get information about a driver.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Drivers** under the Drivers category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            ID of the driver. This can either be the Samsara-specified ID, or an external ID. External IDs are customer specified key-value pairs created in the POST or PATCH requests of this resource. To specify an external ID as part of a path parameter, use the following format: `key:value`. For example, `payrollId:ABFS18600`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DriverResponse
            Returns the specified driver.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.drivers.get(
            id="id",
        )
        """
        _response = self._raw_client.get(id, request_options=request_options)
        return _response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.drivers.delete(
            id="id",
        )
        """
        _response = self._raw_client.delete(id, request_options=request_options)
        return _response.data

    def update(
        self,
        id: str,
        *,
        attributes: typing.Optional[typing.Sequence[UpdateDriverRequestAttributes]] = OMIT,
        carrier_settings: typing.Optional[DriverCarrierSettings] = OMIT,
        current_id_card_code: typing.Optional[str] = OMIT,
        deactivated_at_time: typing.Optional[str] = OMIT,
        driver_activation_status: typing.Optional[UpdateDriverRequestDriverActivationStatus] = OMIT,
        eld_adverse_weather_exemption_enabled: typing.Optional[bool] = OMIT,
        eld_big_day_exemption_enabled: typing.Optional[bool] = OMIT,
        eld_day_start_hour: typing.Optional[int] = OMIT,
        eld_exempt: typing.Optional[bool] = OMIT,
        eld_exempt_reason: typing.Optional[str] = OMIT,
        eld_pc_enabled: typing.Optional[bool] = OMIT,
        eld_ym_enabled: typing.Optional[bool] = OMIT,
        external_ids: typing.Optional[typing.Dict[str, str]] = OMIT,
        has_driving_features_hidden: typing.Optional[DriverHasDrivingFeaturesHidden] = OMIT,
        has_vehicle_unpinning_enabled: typing.Optional[DriverHasVehicleUnpinningEnabled] = OMIT,
        hos_setting: typing.Optional[UpdateDriverRequestHosSetting] = OMIT,
        license_number: typing.Optional[str] = OMIT,
        license_state: typing.Optional[str] = OMIT,
        locale: typing.Optional[UpdateDriverRequestLocale] = OMIT,
        name: typing.Optional[str] = OMIT,
        notes: typing.Optional[str] = OMIT,
        password: typing.Optional[str] = OMIT,
        peer_group_tag_id: typing.Optional[str] = OMIT,
        phone: typing.Optional[str] = OMIT,
        profile_image_base_64: typing.Optional[DriverProfileImageBase64] = OMIT,
        profile_image_url: typing.Optional[DriverProfileImageUrl] = OMIT,
        static_assigned_vehicle_id: typing.Optional[str] = OMIT,
        tachograph_card_number: typing.Optional[str] = OMIT,
        tag_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        us_driver_ruleset_override: typing.Optional[UsDriverRulesetOverride] = OMIT,
        username: typing.Optional[str] = OMIT,
        vehicle_group_tag_id: typing.Optional[str] = OMIT,
        waiting_time_duty_status_enabled: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DriverResponse:
        """
        Update a specific driver's information. This can also be used to activate or de-activate a given driver by setting the driverActivationStatus field. If the driverActivationStatus field is 'deactivated' then the user can also specify the deactivatedAtTime. The deactivatedAtTime cannot be more than 6 months in the past and must not come before the dirver's latest active HOS log. It will be considered an error if deactivatedAtTime is provided with a driverActivationStatus of active.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Drivers** under the Drivers category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            ID of the driver. This can either be the Samsara-specified ID, or an external ID. External IDs are customer specified key-value pairs created in the POST or PATCH requests of this resource. To specify an external ID as part of a path parameter, use the following format: `key:value`. For example, `payrollId:ABFS18600`

        attributes : typing.Optional[typing.Sequence[UpdateDriverRequestAttributes]]

        carrier_settings : typing.Optional[DriverCarrierSettings]

        current_id_card_code : typing.Optional[str]
            The ID Card Code on the back of the physical card assigned to the driver.  Contact Samsara if you would like to enable this feature.

        deactivated_at_time : typing.Optional[str]
            The date and time this driver is considered to be deactivated in RFC 3339 format.

        driver_activation_status : typing.Optional[UpdateDriverRequestDriverActivationStatus]
            A value indicating whether the driver is active or deactivated. Valid values: `active`, `deactivated`.

        eld_adverse_weather_exemption_enabled : typing.Optional[bool]
            Flag indicating this driver may use Adverse Weather exemptions in ELD logs.

        eld_big_day_exemption_enabled : typing.Optional[bool]
            Flag indicating this driver may use Big Day exemption in ELD logs.

        eld_day_start_hour : typing.Optional[int]
            `0` indicating midnight-to-midnight ELD driving hours, `12` to indicate noon-to-noon driving hours.

        eld_exempt : typing.Optional[bool]
            Flag indicating this driver is exempt from the Electronic Logging Mandate.

        eld_exempt_reason : typing.Optional[str]
            Reason that this driver is exempt from the Electronic Logging Mandate (see eldExempt).

        eld_pc_enabled : typing.Optional[bool]
            Flag indicating this driver may select the Personal Conveyance duty status in ELD logs.

        eld_ym_enabled : typing.Optional[bool]
            Flag indicating this driver may select the Yard Move duty status in ELD logs.

        external_ids : typing.Optional[typing.Dict[str, str]]
            The [external IDs](https://developers.samsara.com/docs/external-ids) for the given object.

        has_driving_features_hidden : typing.Optional[DriverHasDrivingFeaturesHidden]

        has_vehicle_unpinning_enabled : typing.Optional[DriverHasVehicleUnpinningEnabled]

        hos_setting : typing.Optional[UpdateDriverRequestHosSetting]

        license_number : typing.Optional[str]
            Driver's state issued license number. The combination of this number and `licenseState` must be unique.

        license_state : typing.Optional[str]
            Abbreviation of US state, Canadian province, or US territory that issued driver's license.

        locale : typing.Optional[UpdateDriverRequestLocale]
            Locale override (uncommon). These are specified by ISO 3166-2 country codes for supported locales. Valid values: `us`, `at`, `be`, `ca`, `gb`, `fr`, `de`, `ie`, `it`, `lu`, `mx`, `nl`, `es`, `ch`, `pr`.

        name : typing.Optional[str]
            Driver's name.

        notes : typing.Optional[str]
            Notes about the driver.

        password : typing.Optional[str]
            Password that the driver can use to login to the Samsara driver app.

        peer_group_tag_id : typing.Optional[str]
            The peer group tag id this driver belong to, leave blank to be in group with everyone, used for gamification.

        phone : typing.Optional[str]
            Phone number of the driver.

        profile_image_base_64 : typing.Optional[DriverProfileImageBase64]

        profile_image_url : typing.Optional[DriverProfileImageUrl]

        static_assigned_vehicle_id : typing.Optional[str]
            ID of vehicle that the driver is permanently assigned to. (uncommon).

        tachograph_card_number : typing.Optional[str]
            Driver's assigned tachograph card number (Europe specific)

        tag_ids : typing.Optional[typing.Sequence[str]]
            IDs of tags the driver is associated with. If your access to the API is scoped by one or more tags, this field is required to pass in.

        timezone : typing.Optional[str]
            Home terminal timezone, in order to indicate what time zone should be used to calculate the ELD logs. Driver timezones use [IANA timezone database](https://www.iana.org/time-zones) keys (e.g. `America/Los_Angeles`, `America/New_York`, `Europe/London`, etc.). You can find a mapping of common timezone formats to IANA timezone keys [here](https://unicode.org/cldr/charts/latest/supplemental/zone_tzid.html).

        us_driver_ruleset_override : typing.Optional[UsDriverRulesetOverride]

        username : typing.Optional[str]
            Driver's login username into the driver app. The username may not contain spaces or the '@' symbol. The username must be unique.

        vehicle_group_tag_id : typing.Optional[str]
            Tag ID which determines which vehicles a driver will see when selecting vehicles.

        waiting_time_duty_status_enabled : typing.Optional[bool]
            Flag indicating this driver may select waiting time duty status in ELD logs

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DriverResponse
            Updated driver object, with ID.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.drivers.update(
            id="id",
        )
        """
        _response = self._raw_client.update(
            id,
            attributes=attributes,
            carrier_settings=carrier_settings,
            current_id_card_code=current_id_card_code,
            deactivated_at_time=deactivated_at_time,
            driver_activation_status=driver_activation_status,
            eld_adverse_weather_exemption_enabled=eld_adverse_weather_exemption_enabled,
            eld_big_day_exemption_enabled=eld_big_day_exemption_enabled,
            eld_day_start_hour=eld_day_start_hour,
            eld_exempt=eld_exempt,
            eld_exempt_reason=eld_exempt_reason,
            eld_pc_enabled=eld_pc_enabled,
            eld_ym_enabled=eld_ym_enabled,
            external_ids=external_ids,
            has_driving_features_hidden=has_driving_features_hidden,
            has_vehicle_unpinning_enabled=has_vehicle_unpinning_enabled,
            hos_setting=hos_setting,
            license_number=license_number,
            license_state=license_state,
            locale=locale,
            name=name,
            notes=notes,
            password=password,
            peer_group_tag_id=peer_group_tag_id,
            phone=phone,
            profile_image_base_64=profile_image_base_64,
            profile_image_url=profile_image_url,
            static_assigned_vehicle_id=static_assigned_vehicle_id,
            tachograph_card_number=tachograph_card_number,
            tag_ids=tag_ids,
            timezone=timezone,
            us_driver_ruleset_override=us_driver_ruleset_override,
            username=username,
            vehicle_group_tag_id=vehicle_group_tag_id,
            waiting_time_duty_status_enabled=waiting_time_duty_status_enabled,
            request_options=request_options,
        )
        return _response.data


class AsyncDriversClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawDriversClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawDriversClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawDriversClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        driver_activation_status: typing.Optional[ListDriversRequestDriverActivationStatus] = None,
        limit: typing.Optional[int] = None,
        after: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        attribute_value_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        attributes: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        updated_after_time: typing.Optional[str] = None,
        created_after_time: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[Driver, ListDriversResponse]:
        """
        Get all drivers in organization.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Drivers** under the Drivers category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        driver_activation_status : typing.Optional[ListDriversRequestDriverActivationStatus]
            If value is `deactivated`, only drivers that are deactivated will appear in the response. This parameter will default to `active` if not provided (fetching only active drivers).

        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 512 objects.

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        parent_tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        attribute_value_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of attribute value IDs. Only entities associated with ALL of the referenced values will be returned (i.e. the intersection of the sets of entities with each value). Example: `attributeValueIds=076efac2-83b5-47aa-ba36-18428436dcac,6707b3f0-23b9-4fe3-b7be-11be34aea544`

        attributes : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data to return entities having given attributes using name-value pair or range query (only for numeric attributes), separated by semicolon. Only entities associated with ALL of the referenced values will be returned (i.e. the intersection of the sets of entities with each value). Example: `attributes=ExampleAttributeName:some_value&attributes=SomeOtherAttr:123&attributes=someNumericAttribute:range(10,20)`

        updated_after_time : typing.Optional[str]
            A filter on data to have an updated at time after or equal to this specified time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        created_after_time : typing.Optional[str]
            A filter on data to have a created at time after or equal to this specified time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[Driver, ListDriversResponse]
            List of all driver objects.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.drivers.list()
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(
            driver_activation_status=driver_activation_status,
            limit=limit,
            after=after,
            parent_tag_ids=parent_tag_ids,
            tag_ids=tag_ids,
            attribute_value_ids=attribute_value_ids,
            attributes=attributes,
            updated_after_time=updated_after_time,
            created_after_time=created_after_time,
            request_options=request_options,
        )

    async def create(
        self,
        *,
        name: str,
        password: str,
        username: str,
        attributes: typing.Optional[typing.Sequence[CreateDriverRequestAttributes]] = OMIT,
        carrier_settings: typing.Optional[DriverCarrierSettings] = OMIT,
        current_id_card_code: typing.Optional[str] = OMIT,
        eld_adverse_weather_exemption_enabled: typing.Optional[bool] = OMIT,
        eld_big_day_exemption_enabled: typing.Optional[bool] = OMIT,
        eld_day_start_hour: typing.Optional[int] = OMIT,
        eld_exempt: typing.Optional[bool] = OMIT,
        eld_exempt_reason: typing.Optional[str] = OMIT,
        eld_pc_enabled: typing.Optional[bool] = OMIT,
        eld_ym_enabled: typing.Optional[bool] = OMIT,
        external_ids: typing.Optional[typing.Dict[str, str]] = OMIT,
        has_driving_features_hidden: typing.Optional[DriverHasDrivingFeaturesHidden] = OMIT,
        has_vehicle_unpinning_enabled: typing.Optional[DriverHasVehicleUnpinningEnabled] = OMIT,
        hos_setting: typing.Optional[DriverHosSetting] = OMIT,
        license_number: typing.Optional[str] = OMIT,
        license_state: typing.Optional[str] = OMIT,
        locale: typing.Optional[CreateDriverRequestLocale] = OMIT,
        notes: typing.Optional[str] = OMIT,
        peer_group_tag_id: typing.Optional[str] = OMIT,
        phone: typing.Optional[str] = OMIT,
        profile_image_base_64: typing.Optional[DriverProfileImageBase64] = OMIT,
        profile_image_url: typing.Optional[DriverProfileImageUrl] = OMIT,
        static_assigned_vehicle_id: typing.Optional[str] = OMIT,
        tachograph_card_number: typing.Optional[str] = OMIT,
        tag_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        us_driver_ruleset_override: typing.Optional[UsDriverRulesetOverride] = OMIT,
        vehicle_group_tag_id: typing.Optional[str] = OMIT,
        waiting_time_duty_status_enabled: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DriverResponse:
        """
        Add a driver to the organization.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Drivers** under the Drivers category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        name : str
            Driver's name.

        password : str
            Password that the driver can use to login to the Samsara driver app.

        username : str
            Driver's login username into the driver app. The username may not contain spaces or the '@' symbol. The username must be unique.

        attributes : typing.Optional[typing.Sequence[CreateDriverRequestAttributes]]

        carrier_settings : typing.Optional[DriverCarrierSettings]

        current_id_card_code : typing.Optional[str]
            The ID Card Code on the back of the physical card assigned to the driver.  Contact Samsara if you would like to enable this feature.

        eld_adverse_weather_exemption_enabled : typing.Optional[bool]
            Flag indicating this driver may use Adverse Weather exemptions in ELD logs.

        eld_big_day_exemption_enabled : typing.Optional[bool]
            Flag indicating this driver may use Big Day exemption in ELD logs.

        eld_day_start_hour : typing.Optional[int]
            `0` indicating midnight-to-midnight ELD driving hours, `12` to indicate noon-to-noon driving hours.

        eld_exempt : typing.Optional[bool]
            Flag indicating this driver is exempt from the Electronic Logging Mandate.

        eld_exempt_reason : typing.Optional[str]
            Reason that this driver is exempt from the Electronic Logging Mandate (see eldExempt).

        eld_pc_enabled : typing.Optional[bool]
            Flag indicating this driver may select the Personal Conveyance duty status in ELD logs.

        eld_ym_enabled : typing.Optional[bool]
            Flag indicating this driver may select the Yard Move duty status in ELD logs.

        external_ids : typing.Optional[typing.Dict[str, str]]
            The [external IDs](https://developers.samsara.com/docs/external-ids) for the given object.

        has_driving_features_hidden : typing.Optional[DriverHasDrivingFeaturesHidden]

        has_vehicle_unpinning_enabled : typing.Optional[DriverHasVehicleUnpinningEnabled]

        hos_setting : typing.Optional[DriverHosSetting]

        license_number : typing.Optional[str]
            Driver's state issued license number. The combination of this number and `licenseState` must be unique.

        license_state : typing.Optional[str]
            Abbreviation of US state, Canadian province, or US territory that issued driver's license.

        locale : typing.Optional[CreateDriverRequestLocale]
            Locale override (uncommon). These are specified by ISO 3166-2 country codes for supported locales. Valid values: `us`, `at`, `be`, `ca`, `gb`, `fr`, `de`, `ie`, `it`, `lu`, `mx`, `nl`, `es`, `ch`, `pr`.

        notes : typing.Optional[str]
            Notes about the driver.

        peer_group_tag_id : typing.Optional[str]
            The peer group tag id this driver belong to, used for gamification.

        phone : typing.Optional[str]
            Phone number of the driver.

        profile_image_base_64 : typing.Optional[DriverProfileImageBase64]

        profile_image_url : typing.Optional[DriverProfileImageUrl]

        static_assigned_vehicle_id : typing.Optional[str]
            ID of vehicle that the driver is permanently assigned to. (uncommon).

        tachograph_card_number : typing.Optional[str]
            Driver's assigned tachograph card number (Europe specific)

        tag_ids : typing.Optional[typing.Sequence[str]]
            IDs of tags the driver is associated with. If your access to the API is scoped by one or more tags, this field is required to pass in.

        timezone : typing.Optional[str]
            Home terminal timezone, in order to indicate what time zone should be used to calculate the ELD logs. Driver timezones use [IANA timezone database](https://www.iana.org/time-zones) keys (e.g. `America/Los_Angeles`, `America/New_York`, `Europe/London`, etc.). You can find a mapping of common timezone formats to IANA timezone keys [here](https://unicode.org/cldr/charts/latest/supplemental/zone_tzid.html).

        us_driver_ruleset_override : typing.Optional[UsDriverRulesetOverride]

        vehicle_group_tag_id : typing.Optional[str]
            Tag ID which determines which vehicles a driver will see when selecting vehicles.

        waiting_time_duty_status_enabled : typing.Optional[bool]
            Flag indicating this driver may select waiting time duty status in ELD logs

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DriverResponse
            Newly created driver object, with Samsara-generated ID.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.drivers.create(
                name="Susan Jones",
                password="aSecurePassword1234",
                username="SusanJones",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            name=name,
            password=password,
            username=username,
            attributes=attributes,
            carrier_settings=carrier_settings,
            current_id_card_code=current_id_card_code,
            eld_adverse_weather_exemption_enabled=eld_adverse_weather_exemption_enabled,
            eld_big_day_exemption_enabled=eld_big_day_exemption_enabled,
            eld_day_start_hour=eld_day_start_hour,
            eld_exempt=eld_exempt,
            eld_exempt_reason=eld_exempt_reason,
            eld_pc_enabled=eld_pc_enabled,
            eld_ym_enabled=eld_ym_enabled,
            external_ids=external_ids,
            has_driving_features_hidden=has_driving_features_hidden,
            has_vehicle_unpinning_enabled=has_vehicle_unpinning_enabled,
            hos_setting=hos_setting,
            license_number=license_number,
            license_state=license_state,
            locale=locale,
            notes=notes,
            peer_group_tag_id=peer_group_tag_id,
            phone=phone,
            profile_image_base_64=profile_image_base_64,
            profile_image_url=profile_image_url,
            static_assigned_vehicle_id=static_assigned_vehicle_id,
            tachograph_card_number=tachograph_card_number,
            tag_ids=tag_ids,
            timezone=timezone,
            us_driver_ruleset_override=us_driver_ruleset_override,
            vehicle_group_tag_id=vehicle_group_tag_id,
            waiting_time_duty_status_enabled=waiting_time_duty_status_enabled,
            request_options=request_options,
        )
        return _response.data

    async def post_driver_remote_signout(
        self, *, driver_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> DriverRemoteSignoutPostDriverRemoteSignoutResponseBody:
        """
        Sign out a driver from the Samsara Driver App

        To access this endpoint, your organization must have the Samsara Platform Premier license.

        Note: Sign out requests made while a logged-in driver does not have internet connection will not log the driver out. A success response will still be provided and the driver will be logged out once they have internet connection.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Driver Remote Signout** under the Drivers category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        driver_id : str
            ID of the driver.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DriverRemoteSignoutPostDriverRemoteSignoutResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.drivers.post_driver_remote_signout(
                driver_id="12434",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.post_driver_remote_signout(
            driver_id=driver_id, request_options=request_options
        )
        return _response.data

    async def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> DriverResponse:
        """
        Get information about a driver.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Drivers** under the Drivers category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            ID of the driver. This can either be the Samsara-specified ID, or an external ID. External IDs are customer specified key-value pairs created in the POST or PATCH requests of this resource. To specify an external ID as part of a path parameter, use the following format: `key:value`. For example, `payrollId:ABFS18600`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DriverResponse
            Returns the specified driver.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.drivers.get(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, request_options=request_options)
        return _response.data

    async def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.drivers.delete(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, request_options=request_options)
        return _response.data

    async def update(
        self,
        id: str,
        *,
        attributes: typing.Optional[typing.Sequence[UpdateDriverRequestAttributes]] = OMIT,
        carrier_settings: typing.Optional[DriverCarrierSettings] = OMIT,
        current_id_card_code: typing.Optional[str] = OMIT,
        deactivated_at_time: typing.Optional[str] = OMIT,
        driver_activation_status: typing.Optional[UpdateDriverRequestDriverActivationStatus] = OMIT,
        eld_adverse_weather_exemption_enabled: typing.Optional[bool] = OMIT,
        eld_big_day_exemption_enabled: typing.Optional[bool] = OMIT,
        eld_day_start_hour: typing.Optional[int] = OMIT,
        eld_exempt: typing.Optional[bool] = OMIT,
        eld_exempt_reason: typing.Optional[str] = OMIT,
        eld_pc_enabled: typing.Optional[bool] = OMIT,
        eld_ym_enabled: typing.Optional[bool] = OMIT,
        external_ids: typing.Optional[typing.Dict[str, str]] = OMIT,
        has_driving_features_hidden: typing.Optional[DriverHasDrivingFeaturesHidden] = OMIT,
        has_vehicle_unpinning_enabled: typing.Optional[DriverHasVehicleUnpinningEnabled] = OMIT,
        hos_setting: typing.Optional[UpdateDriverRequestHosSetting] = OMIT,
        license_number: typing.Optional[str] = OMIT,
        license_state: typing.Optional[str] = OMIT,
        locale: typing.Optional[UpdateDriverRequestLocale] = OMIT,
        name: typing.Optional[str] = OMIT,
        notes: typing.Optional[str] = OMIT,
        password: typing.Optional[str] = OMIT,
        peer_group_tag_id: typing.Optional[str] = OMIT,
        phone: typing.Optional[str] = OMIT,
        profile_image_base_64: typing.Optional[DriverProfileImageBase64] = OMIT,
        profile_image_url: typing.Optional[DriverProfileImageUrl] = OMIT,
        static_assigned_vehicle_id: typing.Optional[str] = OMIT,
        tachograph_card_number: typing.Optional[str] = OMIT,
        tag_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        us_driver_ruleset_override: typing.Optional[UsDriverRulesetOverride] = OMIT,
        username: typing.Optional[str] = OMIT,
        vehicle_group_tag_id: typing.Optional[str] = OMIT,
        waiting_time_duty_status_enabled: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DriverResponse:
        """
        Update a specific driver's information. This can also be used to activate or de-activate a given driver by setting the driverActivationStatus field. If the driverActivationStatus field is 'deactivated' then the user can also specify the deactivatedAtTime. The deactivatedAtTime cannot be more than 6 months in the past and must not come before the dirver's latest active HOS log. It will be considered an error if deactivatedAtTime is provided with a driverActivationStatus of active.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Drivers** under the Drivers category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            ID of the driver. This can either be the Samsara-specified ID, or an external ID. External IDs are customer specified key-value pairs created in the POST or PATCH requests of this resource. To specify an external ID as part of a path parameter, use the following format: `key:value`. For example, `payrollId:ABFS18600`

        attributes : typing.Optional[typing.Sequence[UpdateDriverRequestAttributes]]

        carrier_settings : typing.Optional[DriverCarrierSettings]

        current_id_card_code : typing.Optional[str]
            The ID Card Code on the back of the physical card assigned to the driver.  Contact Samsara if you would like to enable this feature.

        deactivated_at_time : typing.Optional[str]
            The date and time this driver is considered to be deactivated in RFC 3339 format.

        driver_activation_status : typing.Optional[UpdateDriverRequestDriverActivationStatus]
            A value indicating whether the driver is active or deactivated. Valid values: `active`, `deactivated`.

        eld_adverse_weather_exemption_enabled : typing.Optional[bool]
            Flag indicating this driver may use Adverse Weather exemptions in ELD logs.

        eld_big_day_exemption_enabled : typing.Optional[bool]
            Flag indicating this driver may use Big Day exemption in ELD logs.

        eld_day_start_hour : typing.Optional[int]
            `0` indicating midnight-to-midnight ELD driving hours, `12` to indicate noon-to-noon driving hours.

        eld_exempt : typing.Optional[bool]
            Flag indicating this driver is exempt from the Electronic Logging Mandate.

        eld_exempt_reason : typing.Optional[str]
            Reason that this driver is exempt from the Electronic Logging Mandate (see eldExempt).

        eld_pc_enabled : typing.Optional[bool]
            Flag indicating this driver may select the Personal Conveyance duty status in ELD logs.

        eld_ym_enabled : typing.Optional[bool]
            Flag indicating this driver may select the Yard Move duty status in ELD logs.

        external_ids : typing.Optional[typing.Dict[str, str]]
            The [external IDs](https://developers.samsara.com/docs/external-ids) for the given object.

        has_driving_features_hidden : typing.Optional[DriverHasDrivingFeaturesHidden]

        has_vehicle_unpinning_enabled : typing.Optional[DriverHasVehicleUnpinningEnabled]

        hos_setting : typing.Optional[UpdateDriverRequestHosSetting]

        license_number : typing.Optional[str]
            Driver's state issued license number. The combination of this number and `licenseState` must be unique.

        license_state : typing.Optional[str]
            Abbreviation of US state, Canadian province, or US territory that issued driver's license.

        locale : typing.Optional[UpdateDriverRequestLocale]
            Locale override (uncommon). These are specified by ISO 3166-2 country codes for supported locales. Valid values: `us`, `at`, `be`, `ca`, `gb`, `fr`, `de`, `ie`, `it`, `lu`, `mx`, `nl`, `es`, `ch`, `pr`.

        name : typing.Optional[str]
            Driver's name.

        notes : typing.Optional[str]
            Notes about the driver.

        password : typing.Optional[str]
            Password that the driver can use to login to the Samsara driver app.

        peer_group_tag_id : typing.Optional[str]
            The peer group tag id this driver belong to, leave blank to be in group with everyone, used for gamification.

        phone : typing.Optional[str]
            Phone number of the driver.

        profile_image_base_64 : typing.Optional[DriverProfileImageBase64]

        profile_image_url : typing.Optional[DriverProfileImageUrl]

        static_assigned_vehicle_id : typing.Optional[str]
            ID of vehicle that the driver is permanently assigned to. (uncommon).

        tachograph_card_number : typing.Optional[str]
            Driver's assigned tachograph card number (Europe specific)

        tag_ids : typing.Optional[typing.Sequence[str]]
            IDs of tags the driver is associated with. If your access to the API is scoped by one or more tags, this field is required to pass in.

        timezone : typing.Optional[str]
            Home terminal timezone, in order to indicate what time zone should be used to calculate the ELD logs. Driver timezones use [IANA timezone database](https://www.iana.org/time-zones) keys (e.g. `America/Los_Angeles`, `America/New_York`, `Europe/London`, etc.). You can find a mapping of common timezone formats to IANA timezone keys [here](https://unicode.org/cldr/charts/latest/supplemental/zone_tzid.html).

        us_driver_ruleset_override : typing.Optional[UsDriverRulesetOverride]

        username : typing.Optional[str]
            Driver's login username into the driver app. The username may not contain spaces or the '@' symbol. The username must be unique.

        vehicle_group_tag_id : typing.Optional[str]
            Tag ID which determines which vehicles a driver will see when selecting vehicles.

        waiting_time_duty_status_enabled : typing.Optional[bool]
            Flag indicating this driver may select waiting time duty status in ELD logs

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DriverResponse
            Updated driver object, with ID.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.drivers.update(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            id,
            attributes=attributes,
            carrier_settings=carrier_settings,
            current_id_card_code=current_id_card_code,
            deactivated_at_time=deactivated_at_time,
            driver_activation_status=driver_activation_status,
            eld_adverse_weather_exemption_enabled=eld_adverse_weather_exemption_enabled,
            eld_big_day_exemption_enabled=eld_big_day_exemption_enabled,
            eld_day_start_hour=eld_day_start_hour,
            eld_exempt=eld_exempt,
            eld_exempt_reason=eld_exempt_reason,
            eld_pc_enabled=eld_pc_enabled,
            eld_ym_enabled=eld_ym_enabled,
            external_ids=external_ids,
            has_driving_features_hidden=has_driving_features_hidden,
            has_vehicle_unpinning_enabled=has_vehicle_unpinning_enabled,
            hos_setting=hos_setting,
            license_number=license_number,
            license_state=license_state,
            locale=locale,
            name=name,
            notes=notes,
            password=password,
            peer_group_tag_id=peer_group_tag_id,
            phone=phone,
            profile_image_base_64=profile_image_base_64,
            profile_image_url=profile_image_url,
            static_assigned_vehicle_id=static_assigned_vehicle_id,
            tachograph_card_number=tachograph_card_number,
            tag_ids=tag_ids,
            timezone=timezone,
            us_driver_ruleset_override=us_driver_ruleset_override,
            username=username,
            vehicle_group_tag_id=vehicle_group_tag_id,
            waiting_time_duty_status_enabled=waiting_time_duty_status_enabled,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

StreamWorkOrdersRequestWorkOrderStatusesItem = typing.Union[
    typing.Literal[
        "Assigned",
        "Cancelled",
        "Closed",
        "Completed",
        "In Progress",
        "On Hold",
        "Open",
        "Pending Approval",
        "Pending Parts",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_gateway_error import BadGatewayError
from ..errors.gateway_timeout_error import GatewayTimeoutError
from ..errors.internal_server_error import InternalServerError
from ..errors.method_not_allowed_error import MethodNotAllowedError
from ..errors.not_found_error import NotFoundError
from ..errors.not_implemented_error import NotImplementedError
from ..errors.service_unavailable_error import ServiceUnavailableError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.assets_location_link_request_config_object import AssetsLocationLinkRequestConfigObject
from ..types.assets_near_location_link_config_object import AssetsNearLocationLinkConfigObject
from ..types.assets_on_route_link_config_object import AssetsOnRouteLinkConfigObject
from ..types.live_sharing_links_create_live_sharing_link_response_body import (
    LiveSharingLinksCreateLiveSharingLinkResponseBody,
)
from ..types.live_sharing_links_get_live_sharing_links_response_body import (
    LiveSharingLinksGetLiveSharingLinksResponseBody,
)
from ..types.live_sharing_links_update_live_sharing_link_response_body import (
    LiveSharingLinksUpdateLiveSharingLinkResponseBody,
)
from .types.get_live_sharing_links_request_type import GetLiveSharingLinksRequestType
from .types.live_sharing_links_create_live_sharing_link_request_body_type import (
    LiveSharingLinksCreateLiveSharingLinkRequestBodyType,
)

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawLiveSharingLinksClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_live_sharing_links(
        self,
        *,
        ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        type: typing.Optional[GetLiveSharingLinksRequestType] = None,
        limit: typing.Optional[int] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[LiveSharingLinksGetLiveSharingLinksResponseBody]:
        """
        Returns all non-expired Live Sharing Links.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Live Sharing Links** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of Live Share Link IDs

        type : typing.Optional[GetLiveSharingLinksRequestType]
            A filter on the data based on the Live Sharing Link type.  Valid values: `all`, `assetsLocation`, `assetsNearLocation`, `assetsOnRoute`

        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 100 objects.

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[LiveSharingLinksGetLiveSharingLinksResponseBody]
            OK response.
        """
        _response = self._client_wrapper.httpx_client.request(
            "live-shares",
            method="GET",
            params={
                "ids": ids,
                "type": type,
                "limit": limit,
                "after": after,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    LiveSharingLinksGetLiveSharingLinksResponseBody,
                    parse_obj_as(
                        type_=LiveSharingLinksGetLiveSharingLinksResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_live_sharing_link(
        self,
        *,
        name: str,
        type: LiveSharingLinksCreateLiveSharingLinkRequestBodyType,
        assets_location_link_config: typing.Optional[AssetsLocationLinkRequestConfigObject] = OMIT,
        assets_near_location_link_config: typing.Optional[AssetsNearLocationLinkConfigObject] = OMIT,
        assets_on_route_link_config: typing.Optional[AssetsOnRouteLinkConfigObject] = OMIT,
        description: typing.Optional[str] = OMIT,
        expires_at_time: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[LiveSharingLinksCreateLiveSharingLinkResponseBody]:
        """
        Create Live Sharing Link.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Live Sharing Links** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        name : str
            Name of the Live Sharing Link.

        type : LiveSharingLinksCreateLiveSharingLinkRequestBodyType
            Type of the Live Sharing Link. This field specifies which one of '<type>LinkConfig' objects will be used to configure the sharing link.  Valid values: `assetsLocation`, `assetsNearLocation`, `assetsOnRoute`

        assets_location_link_config : typing.Optional[AssetsLocationLinkRequestConfigObject]

        assets_near_location_link_config : typing.Optional[AssetsNearLocationLinkConfigObject]

        assets_on_route_link_config : typing.Optional[AssetsOnRouteLinkConfigObject]

        description : typing.Optional[str]
            Description for the Live Sharing Link (not applicable for 'assetsOnRoute' type).

        expires_at_time : typing.Optional[str]
            Date that this link expires in RFC 3339 format. Can't be set in the past. If not provided then link will never expire.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[LiveSharingLinksCreateLiveSharingLinkResponseBody]
            OK response.
        """
        _response = self._client_wrapper.httpx_client.request(
            "live-shares",
            method="POST",
            json={
                "assetsLocationLinkConfig": convert_and_respect_annotation_metadata(
                    object_=assets_location_link_config,
                    annotation=AssetsLocationLinkRequestConfigObject,
                    direction="write",
                ),
                "assetsNearLocationLinkConfig": convert_and_respect_annotation_metadata(
                    object_=assets_near_location_link_config,
                    annotation=AssetsNearLocationLinkConfigObject,
                    direction="write",
                ),
                "assetsOnRouteLinkConfig": convert_and_respect_annotation_metadata(
                    object_=assets_on_route_link_config, annotation=AssetsOnRouteLinkConfigObject, direction="write"
                ),
                "description": description,
                "expiresAtTime": expires_at_time,
                "name": name,
                "type": type,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    LiveSharingLinksCreateLiveSharingLinkResponseBody,
                    parse_obj_as(
                        type_=LiveSharingLinksCreateLiveSharingLinkResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete_live_sharing_link(
        self, *, id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[None]:
        """
        Delete Live Sharing Link.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Live Sharing Links** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            Unique identifier for the Live Sharing Link.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            "live-shares",
            method="DELETE",
            params={
                "id": id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update_live_sharing_link(
        self,
        *,
        id: str,
        name: str,
        description: typing.Optional[str] = OMIT,
        expires_at_time: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[LiveSharingLinksUpdateLiveSharingLinkResponseBody]:
        """
        Update Live Sharing Link.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Live Sharing Links** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            Unique identifier for the Live Sharing Link.

        name : str
            Name of the Live Sharing Link.

        description : typing.Optional[str]
            Description for the Live Sharing Link (not applicable for 'assetsOnRoute' type).

        expires_at_time : typing.Optional[str]
            Date that this link expires in RFC 3339 format. Can't be set in the past. If not provided then link will never expire.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[LiveSharingLinksUpdateLiveSharingLinkResponseBody]
            OK response.
        """
        _response = self._client_wrapper.httpx_client.request(
            "live-shares",
            method="PATCH",
            params={
                "id": id,
            },
            json={
                "description": description,
                "expiresAtTime": expires_at_time,
                "name": name,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    LiveSharingLinksUpdateLiveSharingLinkResponseBody,
                    parse_obj_as(
                        type_=LiveSharingLinksUpdateLiveSharingLinkResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawLiveSharingLinksClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_live_sharing_links(
        self,
        *,
        ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        type: typing.Optional[GetLiveSharingLinksRequestType] = None,
        limit: typing.Optional[int] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[LiveSharingLinksGetLiveSharingLinksResponseBody]:
        """
        Returns all non-expired Live Sharing Links.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Live Sharing Links** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of Live Share Link IDs

        type : typing.Optional[GetLiveSharingLinksRequestType]
            A filter on the data based on the Live Sharing Link type.  Valid values: `all`, `assetsLocation`, `assetsNearLocation`, `assetsOnRoute`

        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 100 objects.

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[LiveSharingLinksGetLiveSharingLinksResponseBody]
            OK response.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "live-shares",
            method="GET",
            params={
                "ids": ids,
                "type": type,
                "limit": limit,
                "after": after,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    LiveSharingLinksGetLiveSharingLinksResponseBody,
                    parse_obj_as(
                        type_=LiveSharingLinksGetLiveSharingLinksResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_live_sharing_link(
        self,
        *,
        name: str,
        type: LiveSharingLinksCreateLiveSharingLinkRequestBodyType,
        assets_location_link_config: typing.Optional[AssetsLocationLinkRequestConfigObject] = OMIT,
        assets_near_location_link_config: typing.Optional[AssetsNearLocationLinkConfigObject] = OMIT,
        assets_on_route_link_config: typing.Optional[AssetsOnRouteLinkConfigObject] = OMIT,
        description: typing.Optional[str] = OMIT,
        expires_at_time: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[LiveSharingLinksCreateLiveSharingLinkResponseBody]:
        """
        Create Live Sharing Link.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Live Sharing Links** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        name : str
            Name of the Live Sharing Link.

        type : LiveSharingLinksCreateLiveSharingLinkRequestBodyType
            Type of the Live Sharing Link. This field specifies which one of '<type>LinkConfig' objects will be used to configure the sharing link.  Valid values: `assetsLocation`, `assetsNearLocation`, `assetsOnRoute`

        assets_location_link_config : typing.Optional[AssetsLocationLinkRequestConfigObject]

        assets_near_location_link_config : typing.Optional[AssetsNearLocationLinkConfigObject]

        assets_on_route_link_config : typing.Optional[AssetsOnRouteLinkConfigObject]

        description : typing.Optional[str]
            Description for the Live Sharing Link (not applicable for 'assetsOnRoute' type).

        expires_at_time : typing.Optional[str]
            Date that this link expires in RFC 3339 format. Can't be set in the past. If not provided then link will never expire.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[LiveSharingLinksCreateLiveSharingLinkResponseBody]
            OK response.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "live-shares",
            method="POST",
            json={
                "assetsLocationLinkConfig": convert_and_respect_annotation_metadata(
                    object_=assets_location_link_config,
                    annotation=AssetsLocationLinkRequestConfigObject,
                    direction="write",
                ),
                "assetsNearLocationLinkConfig": convert_and_respect_annotation_metadata(
                    object_=assets_near_location_link_config,
                    annotation=AssetsNearLocationLinkConfigObject,
                    direction="write",
                ),
                "assetsOnRouteLinkConfig": convert_and_respect_annotation_metadata(
                    object_=assets_on_route_link_config, annotation=AssetsOnRouteLinkConfigObject, direction="write"
                ),
                "description": description,
                "expiresAtTime": expires_at_time,
                "name": name,
                "type": type,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    LiveSharingLinksCreateLiveSharingLinkResponseBody,
                    parse_obj_as(
                        type_=LiveSharingLinksCreateLiveSharingLinkResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete_live_sharing_link(
        self, *, id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Delete Live Sharing Link.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Live Sharing Links** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            Unique identifier for the Live Sharing Link.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "live-shares",
            method="DELETE",
            params={
                "id": id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update_live_sharing_link(
        self,
        *,
        id: str,
        name: str,
        description: typing.Optional[str] = OMIT,
        expires_at_time: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[LiveSharingLinksUpdateLiveSharingLinkResponseBody]:
        """
        Update Live Sharing Link.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Live Sharing Links** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            Unique identifier for the Live Sharing Link.

        name : str
            Name of the Live Sharing Link.

        description : typing.Optional[str]
            Description for the Live Sharing Link (not applicable for 'assetsOnRoute' type).

        expires_at_time : typing.Optional[str]
            Date that this link expires in RFC 3339 format. Can't be set in the past. If not provided then link will never expire.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[LiveSharingLinksUpdateLiveSharingLinkResponseBody]
            OK response.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "live-shares",
            method="PATCH",
            params={
                "id": id,
            },
            json={
                "description": description,
                "expiresAtTime": expires_at_time,
                "name": name,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    LiveSharingLinksUpdateLiveSharingLinkResponseBody,
                    parse_obj_as(
                        type_=LiveSharingLinksUpdateLiveSharingLinkResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .driver_tiny_response import DriverTinyResponse
from .location import Location
from .safety_event_behavior_labels import SafetyEventBehaviorLabels
from .safety_event_coaching_state import SafetyEventCoachingState
from .safety_event_download_forward_video_url import SafetyEventDownloadForwardVideoUrl
from .safety_event_download_inward_video_url import SafetyEventDownloadInwardVideoUrl
from .safety_event_download_tracked_inward_video_url import SafetyEventDownloadTrackedInwardVideoUrl
from .safety_event_id import SafetyEventId
from .safety_event_max_acceleration_g_force import SafetyEventMaxAccelerationGForce
from .safety_event_time import SafetyEventTime
from .vehicle_tiny_response import VehicleTinyResponse


class SafetyEvent(UniversalBaseModel):
    """
    A safety event.
    """

    behavior_labels: typing_extensions.Annotated[
        typing.Optional[SafetyEventBehaviorLabels],
        FieldMetadata(alias="behaviorLabels"),
        pydantic.Field(alias="behaviorLabels"),
    ] = None
    coaching_state: typing_extensions.Annotated[
        typing.Optional[SafetyEventCoachingState],
        FieldMetadata(alias="coachingState"),
        pydantic.Field(alias="coachingState"),
    ] = None
    download_forward_video_url: typing_extensions.Annotated[
        typing.Optional[SafetyEventDownloadForwardVideoUrl],
        FieldMetadata(alias="downloadForwardVideoUrl"),
        pydantic.Field(alias="downloadForwardVideoUrl"),
    ] = None
    download_inward_video_url: typing_extensions.Annotated[
        typing.Optional[SafetyEventDownloadInwardVideoUrl],
        FieldMetadata(alias="downloadInwardVideoUrl"),
        pydantic.Field(alias="downloadInwardVideoUrl"),
    ] = None
    download_tracked_inward_video_url: typing_extensions.Annotated[
        typing.Optional[SafetyEventDownloadTrackedInwardVideoUrl],
        FieldMetadata(alias="downloadTrackedInwardVideoUrl"),
        pydantic.Field(alias="downloadTrackedInwardVideoUrl"),
    ] = None
    driver: typing.Optional[DriverTinyResponse] = None
    id: typing.Optional[SafetyEventId] = None
    location: typing.Optional[Location] = None
    max_acceleration_g_force: typing_extensions.Annotated[
        typing.Optional[SafetyEventMaxAccelerationGForce],
        FieldMetadata(alias="maxAccelerationGForce"),
        pydantic.Field(alias="maxAccelerationGForce"),
    ] = None
    time: typing.Optional[SafetyEventTime] = None
    vehicle: typing.Optional[VehicleTinyResponse] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class DriverAppSettingsGamificationConfigTinyObjectResponseBody(UniversalBaseModel):
    """
    Gamification configuration for the Driver App.
    """

    anonymize_driver_names: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="anonymizeDriverNames"),
        pydantic.Field(
            alias="anonymizeDriverNames",
            description="Hide the names of other drivers when viewing the driver leaderboard in the mobile app.",
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .fleet_device_object_response_body import FleetDeviceObjectResponseBody
from .industrial_asset_object_response_body import IndustrialAssetObjectResponseBody
from .job_location_response_object_response_body import JobLocationResponseObjectResponseBody
from .job_response_object_response_body_status import JobResponseObjectResponseBodyStatus


class JobResponseObjectResponseBody(UniversalBaseModel):
    """
    Job object to be passed back
    """

    address: JobLocationResponseObjectResponseBody
    created_at: typing_extensions.Annotated[
        str, FieldMetadata(alias="createdAt"), pydantic.Field(alias="createdAt", description="When the job was created")
    ]
    customer_name: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="customerName"),
        pydantic.Field(alias="customerName", description="Customer name for job"),
    ]
    end_date: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="endDate"),
        pydantic.Field(alias="endDate", description="End date of job in RFC 3339 format"),
    ]
    fleet_devices: typing_extensions.Annotated[
        typing.Optional[typing.List[FleetDeviceObjectResponseBody]],
        FieldMetadata(alias="fleetDevices"),
        pydantic.Field(
            alias="fleetDevices",
            description="fleet devices in this job (cannot have both industrial assets and fleet devices in the same job)",
        ),
    ] = None
    id: str = pydantic.Field()
    """
    Job id
    """

    industrial_assets: typing_extensions.Annotated[
        typing.Optional[typing.List[IndustrialAssetObjectResponseBody]],
        FieldMetadata(alias="industrialAssets"),
        pydantic.Field(
            alias="industrialAssets",
            description="Industrial Assets in this job (cannot have both industrial assets and fleet devices in the same job)",
        ),
    ] = None
    modified_at: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="modifiedAt"),
        pydantic.Field(alias="modifiedAt", description="When the job was last modified"),
    ]
    name: str = pydantic.Field()
    """
    Job name
    """

    notes: str = pydantic.Field()
    """
    Notes for the upcoming job
    """

    ontime_window_after_arrival_ms: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="ontimeWindowAfterArrivalMs"),
        pydantic.Field(
            alias="ontimeWindowAfterArrivalMs",
            description="Specifies the time window (in milliseconds) after a stop's scheduled arrival time during which the stop is considered 'on-time'.",
        ),
    ] = None
    ontime_window_before_arrival_ms: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="ontimeWindowBeforeArrivalMs"),
        pydantic.Field(
            alias="ontimeWindowBeforeArrivalMs",
            description="Specifies the time window (in milliseconds) before a stop's scheduled arrival time during which the stop is considered 'on-time'.",
        ),
    ] = None
    start_date: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="startDate"),
        pydantic.Field(alias="startDate", description="Start date of job in RFC 3339 format"),
    ]
    status: JobResponseObjectResponseBodyStatus = pydantic.Field()
    """
    The current job status  Valid values: `active`, `scheduled`, `completed`
    """

    uuid_: typing_extensions.Annotated[
        str, FieldMetadata(alias="uuid"), pydantic.Field(alias="uuid", description="Samsara uuid")
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .device_recovery_recover_asset_request_body_missing_reason import (
        DeviceRecoveryRecoverAssetRequestBodyMissingReason,
    )
    from .device_recovery_recover_asset_request_body_recovery_status import (
        DeviceRecoveryRecoverAssetRequestBodyRecoveryStatus,
    )
    from .device_recovery_recover_asset_request_body_status import DeviceRecoveryRecoverAssetRequestBodyStatus
    from .functions_start_function_run_request_body_params_override import (
        FunctionsStartFunctionRunRequestBodyParamsOverride,
    )
    from .get_assets_inputs_request_type import GetAssetsInputsRequestType
    from .get_driver_efficiency_request_driver_activation_status import GetDriverEfficiencyRequestDriverActivationStatus
    from .get_hos_eld_events_request_driver_activation_status import GetHosEldEventsRequestDriverActivationStatus
    from .get_jobs_request_status import GetJobsRequestStatus
    from .get_qualification_records_stream_request_entity_type import GetQualificationRecordsStreamRequestEntityType
    from .get_qualification_types_request_entity_type import GetQualificationTypesRequestEntityType
    from .get_tag_group_safety_scores_request_score_type import GetTagGroupSafetyScoresRequestScoreType
    from .get_tag_safety_scores_request_score_type import GetTagSafetyScoresRequestScoreType
    from .list_carb_ctc_vehicles_request_test_status_item import ListCarbCtcVehiclesRequestTestStatusItem
    from .ridership_passengers_create_ridership_passenger_request_body_classification import (
        RidershipPassengersCreateRidershipPassengerRequestBodyClassification,
    )
    from .ridership_passengers_update_ridership_passenger_request_body_classification import (
        RidershipPassengersUpdateRidershipPassengerRequestBodyClassification,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "DeviceRecoveryRecoverAssetRequestBodyMissingReason": ".device_recovery_recover_asset_request_body_missing_reason",
    "DeviceRecoveryRecoverAssetRequestBodyRecoveryStatus": ".device_recovery_recover_asset_request_body_recovery_status",
    "DeviceRecoveryRecoverAssetRequestBodyStatus": ".device_recovery_recover_asset_request_body_status",
    "FunctionsStartFunctionRunRequestBodyParamsOverride": ".functions_start_function_run_request_body_params_override",
    "GetAssetsInputsRequestType": ".get_assets_inputs_request_type",
    "GetDriverEfficiencyRequestDriverActivationStatus": ".get_driver_efficiency_request_driver_activation_status",
    "GetHosEldEventsRequestDriverActivationStatus": ".get_hos_eld_events_request_driver_activation_status",
    "GetJobsRequestStatus": ".get_jobs_request_status",
    "GetQualificationRecordsStreamRequestEntityType": ".get_qualification_records_stream_request_entity_type",
    "GetQualificationTypesRequestEntityType": ".get_qualification_types_request_entity_type",
    "GetTagGroupSafetyScoresRequestScoreType": ".get_tag_group_safety_scores_request_score_type",
    "GetTagSafetyScoresRequestScoreType": ".get_tag_safety_scores_request_score_type",
    "ListCarbCtcVehiclesRequestTestStatusItem": ".list_carb_ctc_vehicles_request_test_status_item",
    "RidershipPassengersCreateRidershipPassengerRequestBodyClassification": ".ridership_passengers_create_ridership_passenger_request_body_classification",
    "RidershipPassengersUpdateRidershipPassengerRequestBodyClassification": ".ridership_passengers_update_ridership_passenger_request_body_classification",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "DeviceRecoveryRecoverAssetRequestBodyMissingReason",
    "DeviceRecoveryRecoverAssetRequestBodyRecoveryStatus",
    "DeviceRecoveryRecoverAssetRequestBodyStatus",
    "FunctionsStartFunctionRunRequestBodyParamsOverride",
    "GetAssetsInputsRequestType",
    "GetDriverEfficiencyRequestDriverActivationStatus",
    "GetHosEldEventsRequestDriverActivationStatus",
    "GetJobsRequestStatus",
    "GetQualificationRecordsStreamRequestEntityType",
    "GetQualificationTypesRequestEntityType",
    "GetTagGroupSafetyScoresRequestScoreType",
    "GetTagSafetyScoresRequestScoreType",
    "ListCarbCtcVehiclesRequestTestStatusItem",
    "RidershipPassengersCreateRidershipPassengerRequestBodyClassification",
    "RidershipPassengersUpdateRidershipPassengerRequestBodyClassification",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class WebhooksPostWebhooksUnauthorizedErrorResponseBody(UniversalBaseModel):
    """
    Unauthorized
    """

    message: str = pydantic.Field()
    """
    Message of error
    """

    request_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="requestId"),
        pydantic.Field(
            alias="requestId", description="The request ID; used when reaching out to support for issues with requests."
        ),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.inline_response_2005 import InlineResponse2005
from ..types.inline_response_2006 import InlineResponse2006
from .raw_client import AsyncRawMessagesClient, RawMessagesClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class MessagesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawMessagesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawMessagesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawMessagesClient
        """
        return self._raw_client

    def v_1_get_messages(
        self,
        *,
        end_ms: typing.Optional[int] = None,
        duration_ms: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> InlineResponse2005:
        """
        <n class="warning">
        <nh>
        <i class="fa fa-exclamation-circle"></i>
        This endpoint is still on our legacy API.
        </nh>
        </n>

        Get all messages.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Messages** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        end_ms : typing.Optional[int]
            Time in unix milliseconds that represents the end of time range of messages to return. Used in combination with durationMs. Defaults to now.

        duration_ms : typing.Optional[int]
            Time in milliseconds that represents the duration before endMs to query. Defaults to 24 hours.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InlineResponse2005
            Returns the fetched messages from most recently sent to least recently sent.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.messages.v_1_get_messages()
        """
        _response = self._raw_client.v_1_get_messages(
            end_ms=end_ms, duration_ms=duration_ms, request_options=request_options
        )
        return _response.data

    def v_1_create_messages(
        self, *, driver_ids: typing.Sequence[int], text: str, request_options: typing.Optional[RequestOptions] = None
    ) -> InlineResponse2006:
        """
        <n class="warning">
        <nh>
        <i class="fa fa-exclamation-circle"></i>
        This endpoint is still on our legacy API.
        </nh>
        </n>

        Send a message to a list of driver ids.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Messages** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        driver_ids : typing.Sequence[int]
            IDs of the drivers for whom the messages are sent to.

        text : str
            The text sent in the message. Max 2500 characters allowed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InlineResponse2006
            Returns the created messages.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.messages.v_1_create_messages(
            driver_ids=[111, 222, 333],
            text="This is a message.",
        )
        """
        _response = self._raw_client.v_1_create_messages(
            driver_ids=driver_ids, text=text, request_options=request_options
        )
        return _response.data


class AsyncMessagesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawMessagesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawMessagesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawMessagesClient
        """
        return self._raw_client

    async def v_1_get_messages(
        self,
        *,
        end_ms: typing.Optional[int] = None,
        duration_ms: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> InlineResponse2005:
        """
        <n class="warning">
        <nh>
        <i class="fa fa-exclamation-circle"></i>
        This endpoint is still on our legacy API.
        </nh>
        </n>

        Get all messages.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Messages** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        end_ms : typing.Optional[int]
            Time in unix milliseconds that represents the end of time range of messages to return. Used in combination with durationMs. Defaults to now.

        duration_ms : typing.Optional[int]
            Time in milliseconds that represents the duration before endMs to query. Defaults to 24 hours.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InlineResponse2005
            Returns the fetched messages from most recently sent to least recently sent.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.messages.v_1_get_messages()


        asyncio.run(main())
        """
        _response = await self._raw_client.v_1_get_messages(
            end_ms=end_ms, duration_ms=duration_ms, request_options=request_options
        )
        return _response.data

    async def v_1_create_messages(
        self, *, driver_ids: typing.Sequence[int], text: str, request_options: typing.Optional[RequestOptions] = None
    ) -> InlineResponse2006:
        """
        <n class="warning">
        <nh>
        <i class="fa fa-exclamation-circle"></i>
        This endpoint is still on our legacy API.
        </nh>
        </n>

        Send a message to a list of driver ids.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Messages** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        driver_ids : typing.Sequence[int]
            IDs of the drivers for whom the messages are sent to.

        text : str
            The text sent in the message. Max 2500 characters allowed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InlineResponse2006
            Returns the created messages.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.messages.v_1_create_messages(
                driver_ids=[111, 222, 333],
                text="This is a message.",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.v_1_create_messages(
            driver_ids=driver_ids, text=text, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .dvir_resolved_by_object_response_body_type import DvirResolvedByObjectResponseBodyType


class DvirResolvedByObjectResponseBody(UniversalBaseModel):
    """
    The person who resolved this defect.
    """

    id: str = pydantic.Field()
    """
    ID of the entity that resolved this defect. If the defect was resolved by a driver, this will be a Samsara Driver ID. If the defect was resolved by a mechanic, this will be the Samsara Dashboard User ID of the mechanic.
    """

    name: str = pydantic.Field()
    """
    Name of the person who resolved this defect.
    """

    type: DvirResolvedByObjectResponseBodyType = pydantic.Field()
    """
    Indicates whether this defect was resolved by a driver or a mechanic.  Valid values: `driver`, `mechanic`
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

VehicleSensorConfigurationAreaResponseBodyPosition = typing.Union[typing.Literal["back", "front", "middle"], typing.Any]
